#version 330 

layout(points) in; 
layout(triangle_strip) out; 
layout(max_vertices = 4) out; 

in vec3 vColorPass[]; 
in float fLifeTimePass[]; 
in float fSizePass[]; 
in int iTypePass[]; 
in float fRotationPass[]; 

smooth out vec2 vTexCoord; 
flat out vec4 vColorPart; 

uniform mat4 view;                                                                   
uniform mat4 projection;  
uniform vec3 cameraPosition;

//Generates a rotation matrix
//http://www.neilmendoza.com/glsl-rotation-about-an-arbitrary-axis/
mat4 rotationMatrix(vec3 axis, float angle)
{
    axis = normalize(axis);
    float s = sin(angle);
    float c = cos(angle);
    float oc = 1.0 - c;
    
    return mat4(oc * axis.x * axis.x + c,           oc * axis.x * axis.y - axis.z * s,  oc * axis.z * axis.x + axis.y * s,  0.0,
                oc * axis.x * axis.y + axis.z * s,  oc * axis.y * axis.y + c,           oc * axis.y * axis.z - axis.x * s,  0.0,
                oc * axis.z * axis.x - axis.y * s,  oc * axis.y * axis.z + axis.x * s,  oc * axis.z * axis.z + c,           0.0,
                0.0,                                0.0,                                0.0,                                1.0);
}

void main() 
{ 
	if(iTypePass[0] != 0) {
		vec3 vPosOld = gl_in[0].gl_Position.xyz; 
		float fSize = fSizePass[0]; 
		
		vec3 vPos = gl_in[0].gl_Position.xyz;  
		vec3 toCamera = normalize(cameraPosition - vPos);
		vec3 up = vec3(0,1,0);                                              
		vec3 right = cross(toCamera, up) * fSize;     

		mat4 rotation = rotationMatrix(toCamera, fRotationPass[0]);
		mat4 mVP = projection*view;

		vColorPart = vec4(vColorPass[0], fLifeTimePass[0]); 
        
		vec3 bottomLeft = -(right * 0.5);
		bottomLeft.y -= (fSize * 0.5);

		vec3 bottomRight = bottomLeft + right;

		vec3 upperLeft = bottomLeft;
		upperLeft.y += fSize;

		vec3 upperRight = upperLeft + right;

		vTexCoord = vec2(0.0, 0.0); 
		gl_Position = mVP*(vec4(vPos, 1.0) + vec4(bottomLeft, 1.0) * rotation); 
		EmitVertex(); 
     
		vTexCoord = vec2(0.0, 1.0); 
		gl_Position = mVP*(vec4(vPos, 1.0) + vec4(upperLeft, 1.0) * rotation); 
		EmitVertex(); 
     
		vTexCoord = vec2(1.0, 0.0); 
		gl_Position = mVP*(vec4(vPos, 1.0) + vec4(bottomRight, 1.0) * rotation); 
		EmitVertex(); 
     
		vTexCoord = vec2(1.0, 1.0); 
		gl_Position = mVP*(vec4(vPos, 1.0) + vec4(upperRight, 1.0) * rotation); 
		EmitVertex();
       
		EndPrimitive();
	}
}