#version 330 

layout(points) in; 
layout(points) out; 
layout(max_vertices = 40) out;

in vec3 vPositionPass[]; 
in vec3 vVelocityPass[]; 
in vec3 vColorPass[]; 
in float fLifeTimePass[]; 
in float fSizePass[]; 
in int iTypePass[];
in float fRotationPass[]; 

out vec3 vPositionOut; 
out vec3 vVelocityOut; 
out vec3 vColorOut; 
out float fLifeTimeOut; 
out float fSizeOut; 
out int iTypeOut; 
out float fRotationOut; 

uniform vec3 vOrigin; 
uniform vec3 vGenPositionMin; 
uniform vec3 vGenPositionRange;
uniform vec3 vGenGravityVector; // Gravity vector for particles - updates velocity of particles 
uniform vec3 vGenVelocityMin; // Velocity of new particle - from min to (min+range)
uniform vec3 vGenVelocityRange; 

uniform float fRotationSpeed;
uniform vec3 vGenColor; 
uniform float fGenSize;  

uniform float fGenLifeMin, fGenLifeRange; // Life of new particle - from min to (min+range)
uniform float fTimePassed; // Time passed since last frame

uniform vec3 vRandomSeed; // Seed number for our random number function
vec3 vLocalSeed; 

uniform int iNumToGenerate; // How many particles will be generated next time, if greater than zero, particles are generated

#define DEG2RAD 3.1415926535897932384626433832795 / 180.0

// This function returns random number from zero to one
float randZeroOne() 
{ 
    uint n = floatBitsToUint(vLocalSeed.y * 214013.0 + vLocalSeed.x * 2531011.0 + vLocalSeed.z * 141251.0); 
    n = n * (n * n * 15731u + 789221u); 
    n = (n >> 9u) | 0x3F800000u; 
  
    float fRes =  2.0 - uintBitsToFloat(n); 
    vLocalSeed = vec3(vLocalSeed.x + 147158.0 * fRes, vLocalSeed.y*fRes  + 415161.0 * fRes, vLocalSeed.z + 324154.0*fRes); 
    return fRes; 
}

void main() 
{ 
  vLocalSeed = vRandomSeed;

  vPositionOut = vPositionPass[0]; 
  vVelocityOut = vVelocityPass[0]; 
  fRotationOut = fRotationPass[0];

  vColorOut = vColorPass[0]; 
  fLifeTimeOut = fLifeTimePass[0]-fTimePassed; 
  fSizeOut = fSizePass[0]; 
  iTypeOut = iTypePass[0]; 
  
  if(iTypeOut == 0) 
  {
    EmitVertex(); 
    EndPrimitive();

    for(int i = 0; i < iNumToGenerate; i++) 
    { 
      vPositionOut = vOrigin + vGenPositionMin + vec3(vGenPositionRange.x*randZeroOne(), vGenPositionRange.y*randZeroOne(), vGenPositionRange.z*randZeroOne()); 
      vVelocityOut = vGenVelocityMin+vec3(vGenVelocityRange.x*randZeroOne(), vGenVelocityRange.y*randZeroOne(), vGenVelocityRange.z*randZeroOne()); 
      vColorOut = vGenColor; 
      fLifeTimeOut = fGenLifeMin+fGenLifeRange*randZeroOne(); 
      fSizeOut = fGenSize; 
      iTypeOut = 1; 
	  fRotationOut = DEG2RAD * (360.0 * randZeroOne());
      EmitVertex(); 
      EndPrimitive(); 
    } 
  } 
  else if(fLifeTimeOut > 0.0) 
  { 
	  vPositionOut += vVelocityOut*fTimePassed; 
	  vVelocityOut += vGenGravityVector*fTimePassed; 
	  fRotationOut += DEG2RAD * fRotationSpeed * fTimePassed;

      EmitVertex(); 
      EndPrimitive();  
  } 
}